@@ -22,33 +22,45 @@
 
 #include <config.h>
 
+#include <bonobo/bonobo-main.h>
+
+#include <string.h>
+
 #include "yelp-window.h"
 #include "yelp-section.h"
 #include "yelp-scrollkeeper.h"
-#include "yelp-base.h"
 #include "yelp-man.h"
 #include "yelp-info.h"
+#include "yelp-base.h"
 
 typedef struct {
 	YelpBase     *base;
 	GtkTreeIter *parent;
 } ForeachData;
 
-static void           yelp_base_init          (YelpBase             *base);
-static void           yelp_base_class_init    (YelpBaseClass        *klass);
-static void           yelp_base_new_window_cb (YelpWindow           *window,
-					       YelpBase             *base);
+struct _YelpBasePriv {
+	GNode  *toc_tree;
+
+	GList  *index;
+	GSList *windows;
+};
+
+
+static void           yelp_base_init                (YelpBase       *base);
+static void           yelp_base_class_init          (YelpBaseClass  *klass);
+static void           yelp_base_new_window_cb       (YelpWindow     *window,
+						     YelpBase       *base);
+static void           yelp_base_window_finalized_cb (YelpBase       *base,
+						     YelpWindow     *window);
+
 
 #define PARENT_TYPE BONOBO_OBJECT_TYPE
 static BonoboObjectClass *parent_class;
 
-struct _YelpBasePriv {
-        GtkTreeStore  *content_store;
-};
-
 static void
-impl_Yelp_newWindow (PortableServer_Servant   servant,
-		     CORBA_Environment       *ev)
+impl_Yelp_newWindow (PortableServer_Servant  servant,
+		     const CORBA_char       *url,
+		     CORBA_Environment      *ev)
 {
 	YelpBase  *yelp_base;
 	GtkWidget *window;
@@ -56,22 +68,55 @@ impl_Yelp_newWindow (PortableServer_Servant   servant,
 	yelp_base = YELP_BASE (bonobo_object (servant));
 	
 	window = yelp_base_new_window (yelp_base);
+
 	gtk_widget_show_all (window);
+
+	if (url && strcmp (url, "")) {
+		yelp_window_open_uri (YELP_WINDOW (window), url);
+	}
+}
+
+static GNOME_Yelp_WindowList *
+impl_Yelp_getWindows (PortableServer_Servant  servant,
+		      CORBA_Environment      *ev)
+{
+	YelpBase              *base;
+	YelpBasePriv          *priv;
+	GNOME_Yelp_WindowList *list;
+	gint                   len, i;
+	GSList                *node;
+	const gchar           *url;
+	
+	base = YELP_BASE (bonobo_object (servant));
+	priv = base->priv;
+
+	len  = g_slist_length (priv->windows);
+	
+	list = GNOME_Yelp_WindowList__alloc ();
+	list->_buffer = CORBA_sequence_CORBA_string_allocbuf (len);
+	list->_length = len;
+	list->_maximum = len;
+	CORBA_sequence_set_release (list, CORBA_TRUE);
+	
+	for (node = priv->windows, i = 0; node; node = node->next, i++) {
+		url = yelp_window_get_current_uri (YELP_WINDOW (node->data));
+		list->_buffer[i] = CORBA_string_dup (url);
+	}
+	
+	return list;
 }
 
 static void
 yelp_base_init (YelpBase *base)
 {
-        YelpBasePriv   *priv;
+        YelpBasePriv *priv;
 
         priv = g_new0 (YelpBasePriv, 1);
         
-        priv->content_store  = gtk_tree_store_new (3, 
-						   G_TYPE_STRING, 
-						   G_TYPE_POINTER,
-						   G_TYPE_BOOLEAN);
-        
-        base->priv       = priv;
+	priv->toc_tree = g_node_new (NULL);
+	priv->index    = NULL;
+	priv->windows  = NULL;
+        base->priv     = priv;
 }
 
 static void
@@ -81,7 +126,8 @@ yelp_base_class_init (YelpBaseClass *klass)
 	
 	parent_class = gtk_type_class (PARENT_TYPE);
 
-	epv->newWindow = impl_Yelp_newWindow;
+	epv->newWindow  = impl_Yelp_newWindow;
+	epv->getWindows = impl_Yelp_getWindows;
 }
 
 static void
@@ -97,31 +143,56 @@ yelp_base_new_window_cb (YelpWindow *window, YelpBase *base)
 	gtk_widget_show_all (new_window);
 }
 
+static void
+yelp_base_window_finalized_cb (YelpBase *base, YelpWindow *window)
+{
+	YelpBasePriv *priv;
+	
+	g_return_if_fail (YELP_IS_BASE (base));
+	
+	priv = base->priv;
+	
+	priv->windows = g_slist_remove (priv->windows, window);
+
+	if (g_slist_length (priv->windows) == 0) {
+		bonobo_main_quit ();
+	}
+}
+
 YelpBase *
 yelp_base_new (void)
 {
-        YelpBase *base;
-	gboolean  result;
-
+        YelpBase     *base;
+	YelpBasePriv *priv;
+	
         base = g_object_new (YELP_TYPE_BASE, NULL);
-
-	result = yelp_scrollkeeper_init (base->priv->content_store);
-	result = yelp_man_init (base->priv->content_store);
-	result = yelp_info_init (base->priv->content_store);
-
+	priv = base->priv;
+	
+	yelp_scrollkeeper_init (priv->toc_tree, &priv->index);
+	yelp_man_init (base->priv->toc_tree, &priv->index);
+	yelp_info_init (base->priv->toc_tree, &priv->index);
+	
         return base;
 }
 
 GtkWidget *
 yelp_base_new_window (YelpBase *base)
 {
-	GtkWidget *window;
+	YelpBasePriv *priv;
+	GtkWidget    *window;
         
         g_return_val_if_fail (YELP_IS_BASE (base), NULL);
+
+	priv = base->priv;
         
-	/* FIXME: Have the windows in a list */
-        window = yelp_window_new (GTK_TREE_MODEL (base->priv->content_store));
+        window = yelp_window_new (priv->toc_tree, priv->index);
         
+	priv->windows = g_slist_prepend (priv->windows, window);
+
+	g_object_weak_ref (G_OBJECT (window),
+			   (GWeakNotify) yelp_base_window_finalized_cb,
+			   base);
+	
 	g_signal_connect (window, "new_window_requested",
 			  G_CALLBACK (yelp_base_new_window_cb),
 			  base);
