@@ -44,7 +44,6 @@ static void yvc_tree_selection_changed_cb (GtkTreeSelection        *selection,
 
 enum {
 	URL_SELECTED,
-	TITLE_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -117,7 +116,7 @@ yvc_init (YelpViewContent *view)
 
 	priv->content_tree = gtk_tree_view_new ();
 	priv->tree_store   = gtk_tree_store_new (2,
-						 G_TYPE_STRING,
+						 G_TYPE_STRING, 
 						 G_TYPE_POINTER);
 	
 	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->content_tree),
@@ -144,17 +143,6 @@ yvc_class_init (YelpViewContentClass *klass)
 			      yelp_marshal_VOID__STRING_STRING_BOOLEAN,
 			      G_TYPE_NONE,
 			      3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
-
-	signals[TITLE_CHANGED] = 
-		g_signal_new ("title_changed",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (YelpViewContentClass,
-					       title_changed),
-			      NULL, NULL,
-			      yelp_marshal_VOID__STRING,
-			      G_TYPE_NONE,
-			      1, G_TYPE_STRING);
 }
 
 static void
@@ -243,9 +231,9 @@ yelp_view_content_insert_tree (YelpViewContent *content,
 			       GtkTreeIter     *parent,
 			       GNode           *node)
 {
-	GtkTreeIter iter;
+	GtkTreeIter  iter;
 	YelpSection *section;
-	GNode *child;
+	GNode       *child;
 	
 	gtk_tree_store_append (content->priv->tree_store,
 			       &iter, parent);
@@ -270,7 +258,7 @@ static void
 yelp_view_content_set_tree (YelpViewContent *content,
 			    GNode           *node)
 {
-	GNode               *child;
+	GNode *child;
 
 	g_return_if_fail (YELP_IS_VIEW_CONTENT (content));
 	g_return_if_fail (node != NULL);
@@ -290,7 +278,6 @@ yelp_view_content_show_uri (YelpViewContent *content,
 			    const gchar     *url)
 {
 	YelpViewContentPriv *priv;
-	YelpSection         *section;
 	gchar               *content_url;
 	GNode               *node;
 	
@@ -323,7 +310,7 @@ yelp_view_content_show_uri (YelpViewContent *content,
 			/* Try to find it in the scrollkeeper database, 
 			   doesn't have to exist here */
 			node = yelp_scrollkeeper_get_toc_tree (docpath);
-
+			
 			if (node) {
 				yelp_view_content_set_tree (content, node);
 				
@@ -351,15 +338,10 @@ yelp_view_content_show_uri (YelpViewContent *content,
 		content_url = (char *) url;
 	}
 
-	/* FIXME: This is a quite dubious way to load the url... */
-	section = yelp_section_new (YELP_SECTION_DOCUMENT,
-				    NULL, content_url, NULL, NULL);
-
-	yelp_html_open_section (YELP_HTML (content->priv->html_view), section);
-	yelp_section_free (section);
+	yelp_html_open_uri (YELP_HTML (priv->html_view), 
+			    content_url, NULL);
 
 	if (content_url != url) {
 		g_free (content_url);
 	}
 }
-
