@@ -57,16 +57,20 @@ enum {
 
 static gint signals[LAST_SIGNAL] = { 0 };
 
-struct YelpImportantDocsSection {
+typedef struct {
 	char *title;
 	GList *seriesids;
-};
+} YelpImportantDocsSection;
+
+#define BUFFER_SIZE 4096
 
 struct _YelpViewTOCPriv {
 	GtkWidget    *html_view;
 	HtmlDocument *doc;
 	GNode        *doc_tree;
 	GList        *important_sections;
+	char          buffer[BUFFER_SIZE];
+	int           buffer_pos;
 };
 
 GType
@@ -145,18 +149,29 @@ static void
 yelp_view_toc_open (YelpViewTOC *view)
 {
 	html_document_open_stream (view->priv->doc, "text/html");
+	view->priv->buffer_pos = 0;
 }
 
 static void
 yelp_view_toc_close (YelpViewTOC *view)
 {
-	/* TODO: If buffering, flush buffers */
+	YelpViewTOCPriv *priv = view->priv;
+	
+	if (priv->buffer_pos > 0) {
+		html_document_write_stream (priv->doc, priv->buffer, priv->buffer_pos);
+		priv->buffer_pos = 0;
+	}
+
+		
 	html_document_close_stream (view->priv->doc);
 }
 
 static void
 yelp_view_toc_write (YelpViewTOC *view, char *data, int len)
 {
+	YelpViewTOCPriv *priv = view->priv;
+	int chunk_size;
+	
 	if (len < 0) {
 		len = strlen (data);
 	}
@@ -165,9 +180,20 @@ yelp_view_toc_write (YelpViewTOC *view, char *data, int len)
 	g_print ("%.*s", len,data);
 #endif
 
-	/* TODO: Maybe we should be buffering writes */
-	
-	html_document_write_stream (view->priv->doc, data, len);
+	while (len > 0) {
+		chunk_size = MIN (BUFFER_SIZE - priv->buffer_pos, len);
+		
+		memcpy (priv->buffer + priv->buffer_pos, data, chunk_size);
+		priv->buffer_pos += chunk_size;
+		len -= chunk_size;
+		data += chunk_size;
+
+		if (priv->buffer_pos == BUFFER_SIZE) {
+			html_document_write_stream (priv->doc, priv->buffer, BUFFER_SIZE);
+			priv->buffer_pos = 0;
+		}
+		
+	}
 }
 
 static void
@@ -193,10 +219,10 @@ yelp_view_toc_write_header (YelpViewTOC *view, char *title)
 	char *header="\n"
 "<html>\n"
 "  <head>\n"
+"    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
 "    <title>\n"
 "      %s\n"
 "    </title> \n"
-"    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
 "    <style type=\"text/css\">\n"
 "      A:link { color: #00008b }\n"
 "      A:visited { color: #00008b }\n"
@@ -242,12 +268,15 @@ yelp_view_toc_start (YelpViewTOC *view)
 	GNode           *root;
 	char            *path;
 	GList           *sections, *seriesids;
-	gchar           *other_docs_string = _("Other document systems");
+	gchar           *other_docs_string = _("Other Documents");
 	gchar           *man_string = _("Manual pages");
 	gchar           *info_string = _("Info pages");
 	gchar           *installed_string = _("Installed documents");
-	struct YelpImportantDocsSection *important_section;
-	
+	YelpImportantDocsSection *important_section;
+	gboolean         important_doc_installed = FALSE;
+	gboolean         man_installed = FALSE;
+	gboolean         info_installed = FALSE;
+      
 	priv = view->priv;
 
 	if (!g_node_first_child (priv->doc_tree)) {
@@ -266,16 +295,36 @@ yelp_view_toc_start (YelpViewTOC *view)
 	sections = priv->important_sections;
 	while (sections != NULL) {
 		important_section = sections->data;
-		
-		yelp_view_toc_printf (view, 
-				      "<h2>%s</h2>\n"
-				      "<ul>\n", important_section->title);
 
 		seriesids = important_section->seriesids;
+
+		/* Check if any of the important documents are installed  *
+		 * before trying to write the section topic               */
+
+		for (seriesids = important_section->seriesids; 
+		     seriesids; 
+		     seriesids = seriesids->next) {
+			seriesid = seriesids->data;
+
+			if (yelp_scrollkeeper_lookup_seriesid (seriesid)){
+				important_doc_installed = TRUE;
+			}
+		}
+		
+ 		if (important_doc_installed) {
+			yelp_view_toc_printf (view, 
+					      "<h2>%s</h2>\n"
+					      "<ul>\n", 
+					      important_section->title);
+		}
+		
+		seriesids = important_section->seriesids;
+		
 		while (seriesids != NULL) {
 			seriesid = seriesids->data;
 
 			node = yelp_scrollkeeper_lookup_seriesid (seriesid);
+
 			if (node) {
 				section = node->data;
 				yelp_view_toc_printf (view, 
@@ -305,7 +354,7 @@ yelp_view_toc_start (YelpViewTOC *view)
 	node = g_node_first_child (root);
 
 	while (node) {
-		section = (YelpSection *) node->data;
+		section = YELP_SECTION (node->data);
 		path = yelp_util_node_to_string_path (node);
 		yelp_view_toc_printf (view, 
 				      "<a href=\"toc:scrollkeeper/%s\">%s</a><br>\n", 
@@ -315,13 +364,32 @@ yelp_view_toc_start (YelpViewTOC *view)
 		node = g_node_next_sibling (node);
 	}
 
+	if (yelp_util_find_toplevel (priv->doc_tree, "man")) {
+		man_installed = TRUE;
+	}
+	
+	if (yelp_util_find_toplevel (priv->doc_tree, "info")) {
+		info_installed = TRUE;
+	}
+	
+	
+	if (man_installed || info_installed) {
+		yelp_view_toc_printf (view, "<h2>%s</h2>\n", 
+				      other_docs_string);
+		
+		if (man_installed) {
+			yelp_view_toc_printf (view, 
+					      "<a href=\"toc:man\">%s</a><br>\n",
+					      man_string);
+		}
+		
+		if (info_installed) {
+			yelp_view_toc_printf (view,
+					      "<a href=\"toc:info\">%s</a><br>\n",
+					      info_string);
+		}
+	}
 	
-	yelp_view_toc_printf (view, 
-			     "<h2>%s</h2>\n"
-			     "<a href=\"toc:man\">%s</a><br>\n"
-			     "<a href=\"toc:info\">%s</a><br>\n",
-			      other_docs_string, man_string, info_string);
-			      
 	yelp_view_toc_write (view, 
 			     "</td>\n", -1);
 
@@ -427,11 +495,11 @@ yelp_view_toc_man_emit (YelpViewTOC *view, GNode *first)
 
 static void 
 yelp_view_toc_man_2 (YelpViewTOC *view,
-		     GNode *root)
+		     GNode       *root)
 {
 	GNode *first;
 	gchar *name;
-	gchar *string = _("<a href=\"toc:man\">Manual pages</a>: ");
+	gchar *string = _("Manual pages");
 	
 	if (root->children == NULL) {
 		return;
@@ -441,11 +509,11 @@ yelp_view_toc_man_2 (YelpViewTOC *view,
 
 	yelp_view_toc_open (view);
 	
-	yelp_view_toc_write_header (view, "Manual pages");
+	yelp_view_toc_write_header (view, string);
 		
 	name = yelp_view_toc_full_path_name (view, root);
 	
-	yelp_view_toc_printf (view, "<h1>%s '%s'</h1>\n", string, name);
+	yelp_view_toc_printf (view, "<h1><a href=\"toc:man\">%s</a>: '%s'</h1>\n", string, name);
 	g_free (name);
 
 	yelp_view_toc_man_emit (view, first);
@@ -461,7 +529,7 @@ yelp_view_toc_man_1 (YelpViewTOC *view)
 	GNode           *root, *node, *child;
 	YelpSection     *section;
 	char            *path;
-	gchar           *string = _("Manual page sections");
+	gchar           *string = _("Manual Pages");
 	
 	priv = view->priv;
 
@@ -480,7 +548,7 @@ yelp_view_toc_man_1 (YelpViewTOC *view)
 
 	yelp_view_toc_open (view);
 	
-	yelp_view_toc_write_header (view, _("Manual page sections"));
+	yelp_view_toc_write_header (view, string);
 	
 	yelp_view_toc_printf (view, "<h1>%s</h1>\n", string);
 	
@@ -488,7 +556,7 @@ yelp_view_toc_man_1 (YelpViewTOC *view)
 		child = g_node_first_child (node);
 		
 		if (child) {
-			section = (YelpSection *) node->data;
+			section = YELP_SECTION (node->data);
 			path = yelp_util_node_to_string_path (node);
 			yelp_view_toc_printf (view, 
 					      "<a href=\"toc:man/%s\">%s</a><br>\n", 
@@ -532,7 +600,7 @@ yelp_view_toc_info (YelpViewTOC *view)
 	yelp_view_toc_printf (view, "<h1>%s</h1>\n", string);
 	
 	do {
-		section = (YelpSection *) node->data;
+		section = YELP_SECTION (node->data);
 		url = yelp_util_compose_path_url (root, section->uri);
 		yelp_view_toc_printf (view, 
 				      "<a href=\"%s\">%s</a><br>\n", 
@@ -553,7 +621,7 @@ yelp_view_read_important_docs (YelpViewTOC *view)
 	xmlNodePtr section;
 	xmlNodePtr title;
 	xmlChar *prop;
-	struct YelpImportantDocsSection *important_section;
+	YelpImportantDocsSection *important_section;
 	
 	doc = xmlParseFile (DATADIR "/yelp/important_docs.xml");
 	if (doc == NULL)
@@ -573,8 +641,7 @@ yelp_view_read_important_docs (YelpViewTOC *view)
 	section = node->children;
 	while (section) {
 		if (strcmp (section->name, "section") == 0) {
-			important_section = g_new0 (struct YelpImportantDocsSection, 1);
-			
+			important_section = g_new0 (YelpImportantDocsSection, 1);
 			child = section->children;
 			while (child) {
 				if (strcmp (child->name, "title") == 0) {
@@ -739,4 +806,7 @@ yelp_view_toc_open_url (YelpViewTOC *view, const char *url)
 	} else {
 		g_warning ("Unknown toc type %s\n", url);
 	}
+
+ 	gtk_adjustment_set_value (
+ 		gtk_layout_get_vadjustment (GTK_LAYOUT (view)), 0);
 }
