@@ -138,7 +138,7 @@ static GtkItemFactoryEntry menu_items[] = {
 	{N_("/_File"),              NULL,         0,                  0,                           "<Branch>"},
 	{N_("/File/_New window"),   NULL,         yw_new_window_cb,   0,                           "<StockItem>", GTK_STOCK_NEW     },
 	{N_("/File/_Close window"), NULL,         yw_close_window_cb, 0,                           "<StockItem>", GTK_STOCK_CLOSE   },
-	{N_("/File/_Quit"),         NULL,         yw_exit_cb,         0,                           "<StockItem>", GTK_STOCK_QUIT    },
+/*	{N_("/File/_Quit"),         NULL,         yw_exit_cb,         0,                           "<StockItem>", GTK_STOCK_QUIT    }, */
 	{N_("/_Go"),                NULL,         0,                  0,                           "<Branch>"},
 	{N_("/Go/_Back"),           NULL,         yw_history_go_cb,   YELP_WINDOW_ACTION_BACK,     "<StockItem>", GTK_STOCK_GO_BACK    },
 	{N_("/Go/_Forward"),        NULL,         yw_history_go_cb,   YELP_WINDOW_ACTION_FORWARD,  "<StockItem>", GTK_STOCK_GO_FORWARD },
@@ -315,10 +315,10 @@ yw_handle_url (YelpWindow *window, const gchar *url)
 		   strncmp (url, "info:", 5) == 0 ||
 		   strncmp (url, "ghelp:", 6) == 0 ||
 		   strncmp (url, "path:", 5) == 0) {
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
-					       PAGE_CONTENT_VIEW);
 		yelp_view_content_show_uri (YELP_VIEW_CONTENT (priv->content_view),
 					    url);
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
+					       PAGE_CONTENT_VIEW);
 		return TRUE;
 	} else if (strncmp (url, "index:", 6) == 0) {
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
